@@ -41,6 +41,56 @@ the very first instruction (or even the start of ``main``), you instead should u
 When you use :func:`debug`, the return value is a :class:`.tube` object
 that you interact with exactly like normal.
 
+Using GDB Python API
+~~~~~~~~~~~~~~~~~~~~
+
+GDB provides Python API, which is documented at
+https://sourceware.org/gdb/onlinedocs/gdb/Python-API.html. Pwntools allows you
+to call it right from the exploit, without having to write a gdbscript. This is
+useful for inspecting program state, e.g. asserting that leaked values are
+correct, or that certain packets trigger a particular code path or put the heap
+in a desired state.
+
+Pass ``api=True`` to :func:`attach` or :func:`debug` in order to enable GDB
+Python API access. Pwntools will then connect to GDB using RPyC library:
+https://rpyc.readthedocs.io/en/latest/.
+
+At the moment this is an experimental feature with the following limitations:
+
+- Only Python 3 is supported.
+
+  Well, technically that's not quite true. The real limitation is that your
+  GDB's Python interpreter major version should be the same as that of
+  Pwntools. However, most GDBs use Python 3 nowadays.
+
+  Different minor versions are allowed as long as no incompatible values are
+  sent in either direction. See
+  https://rpyc.readthedocs.io/en/latest/install.html#cross-interpreter-compatibility
+  for more information.
+
+  Use
+
+  ::
+
+      $ gdb -batch -ex 'python import sys; print(sys.version)'
+
+  in order to check your GDB's Python version.
+- If your GDB uses a different Python interpreter than Pwntools (for example,
+  because you run Pwntools out of a virtualenv), you should install ``rpyc``
+  package into its ``sys.path``. Use
+
+  ::
+
+      $ gdb -batch -ex 'python import rpyc'
+
+  in order to check whether this is necessary.
+- Only local processes are supported.
+- It is not possible to tell whether ``gdb.execute('continue')`` will be
+  executed synchronously or asynchronously (in gdbscripts it is always
+  synchronous). Therefore it is recommended to use either the explicitly
+  synchronous :func:`pwnlib.gdb.Gdb.continue_and_wait` or the explicitly
+  asynchronous :func:`pwnlib.gdb.Gdb.continue_nowait` instead.
+
 Tips and Troubleshooting
 ------------------------
 
@@ -91,14 +141,22 @@ Member Documentation
 from __future__ import absolute_import
 from __future__ import division
 
+from contextlib import contextmanager
 import os
+import platform
 import random
 import re
 import shlex
 import six
+import six.moves
+import socket
 import tempfile
+from threading import Event
 import time
 
+from rpyc import BgServingThread
+from rpyc.utils.factory import unix_connect
+
 from pwnlib import adb
 from pwnlib import atexit
 from pwnlib import elf
@@ -117,8 +175,8 @@ from pwnlib.util import proc
 log = getLogger(__name__)
 
 @LocalContext
-def debug_assembly(asm, gdbscript=None, vma=None):
-    r"""debug_assembly(asm, gdbscript=None, vma=None) -> tube
+def debug_assembly(asm, gdbscript=None, vma=None, api=False):
+    r"""debug_assembly(asm, gdbscript=None, vma=None, api=False) -> tube
 
     Creates an ELF file, and launches it under a debugger.
 
@@ -130,6 +188,7 @@ def debug_assembly(asm, gdbscript=None, vma=None):
         asm(str): Assembly code to debug
         gdbscript(str): Script to run in GDB
         vma(int): Base address to load the shellcode at
+        api(bool): Enable access to GDB Python API
         \**kwargs: Override any :obj:`pwnlib.context.context` values.
 
     Returns:
@@ -152,17 +211,18 @@ def debug_assembly(asm, gdbscript=None, vma=None):
         adb.push(tmp_elf, android_path)
         tmp_elf = android_path
 
-    return debug(tmp_elf, gdbscript=gdbscript, arch=context.arch)
+    return debug(tmp_elf, gdbscript=gdbscript, arch=context.arch, api=api)
 
 @LocalContext
-def debug_shellcode(data, gdbscript=None, vma=None):
-    r"""debug_shellcode(data, gdbscript=None, vma=None) -> tube
+def debug_shellcode(data, gdbscript=None, vma=None, api=False):
+    r"""debug_shellcode(data, gdbscript=None, vma=None, api=False) -> tube
     Creates an ELF file, and launches it under a debugger.
 
     Arguments:
         data(str): Assembled shellcode bytes
         gdbscript(str): Script to run in GDB
         vma(int): Base address to load the shellcode at
+        api(bool): Enable access to GDB Python API
         \**kwargs: Override any :obj:`pwnlib.context.context` values.
 
     Returns:
@@ -188,7 +248,7 @@ def debug_shellcode(data, gdbscript=None, vma=None):
         adb.push(tmp_elf, android_path)
         tmp_elf = android_path
 
-    return debug(tmp_elf, gdbscript=gdbscript, arch=context.arch)
+    return debug(tmp_elf, gdbscript=gdbscript, arch=context.arch, api=api)
 
 def _gdbserver_args(pid=None, path=None, args=None, which=None, env=None):
     """_gdbserver_args(pid=None, path=None, args=None, which=None, env=None) -> list
@@ -304,7 +364,7 @@ def _get_runner(ssh=None):
     else:                          return tubes.process.process
 
 @LocalContext
-def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, **kwargs):
+def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, api=False, **kwargs):
     r"""
     Launch a GDB server with the specified command line,
     and launches GDB to attach to it.
@@ -317,9 +377,12 @@ def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, **kw
         ssh(:class:`.ssh`): Remote ssh session to use to launch the process.
         sysroot(str): Foreign-architecture sysroot, used for QEMU-emulated binaries
             and Android targets.
+        api(bool): Enable access to GDB Python API.
 
     Returns:
-        :class:`.process` or :class:`.ssh_channel`: A tube connected to the target process
+        :class:`.process` or :class:`.ssh_channel`: A tube connected to the target process.
+        When ``api=True``, ``gdb`` member of the returned object contains a :class:`Gdb`
+        instance.
 
     Notes:
 
@@ -353,55 +416,98 @@ def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, **kw
 
     Examples:
 
-    >>> # Create a new process, and stop it at 'main'
-    >>> io = gdb.debug('bash', '''
-    ... break main
-    ... continue
-    ... ''')
-    >>> # Send a command to Bash
-    >>> io.sendline("echo hello")
-    >>> io.recvline()
-    b'hello\n'
-    >>> # Interact with the process
-    >>> io.interactive() # doctest: +SKIP
-    >>> io.close()
+        Create a new process, and stop it at 'main'
 
-    >>> # Create a new process, and stop it at '_start'
-    >>> io = gdb.debug('bash', '''
-    ... # Wait until we hit the main executable's entry point
-    ... break _start
-    ... continue
-    ...
-    ... # Now set breakpoint on shared library routines
-    ... break malloc
-    ... break free
-    ... continue
-    ... ''')
-    >>> # Send a command to Bash
-    >>> io.sendline("echo hello")
-    >>> io.recvline()
-    b'hello\n'
-    >>> # Interact with the process
-    >>> io.interactive() # doctest: +SKIP
-    >>> io.close()
+        >>> io = gdb.debug('bash', '''
+        ... break main
+        ... continue
+        ... ''')
 
-    You can use :func:`debug` to spawn new processes on remote machines as well,
-    by using the ``ssh=`` keyword to pass in your :class:`.ssh` instance.
+        Send a command to Bash
 
-    >>> # Connect to the SSH server
-    >>> # Start a process on the server
-    >>> shell = ssh('travis', 'example.pwnme', password='demopass')
-    >>> io = gdb.debug(['bash'],
-    ...                 ssh = shell,
-    ...                 gdbscript = '''
-    ... break main
-    ... continue
-    ... ''')
-    >>> # Send a command to Bash
-    >>> io.sendline("echo hello")
-    >>> # Interact with the process
-    >>> io.interactive() # doctest: +SKIP
-    >>> io.close()
+        >>> io.sendline("echo hello")
+        >>> io.recvline()
+        b'hello\n'
+
+        Interact with the process
+
+        >>> io.interactive() # doctest: +SKIP
+        >>> io.close()
+
+        Create a new process, and stop it at '_start'
+
+        >>> io = gdb.debug('bash', '''
+        ... # Wait until we hit the main executable's entry point
+        ... break _start
+        ... continue
+        ...
+        ... # Now set breakpoint on shared library routines
+        ... break malloc
+        ... break free
+        ... continue
+        ... ''')
+
+        Send a command to Bash
+
+        >>> io.sendline("echo hello")
+        >>> io.recvline()
+        b'hello\n'
+
+        Interact with the process
+
+        >>> io.interactive() # doctest: +SKIP
+        >>> io.close()
+
+    Using GDB Python API:
+
+    .. doctest
+       :skipif: six.PY2
+
+        Debug a new process
+
+        >>> io = gdb.debug(['echo', 'foo'], api=True)
+
+        Stop at 'write'
+
+        >>> bp = io.gdb.Breakpoint('write', temporary=True)
+        >>> io.gdb.continue_and_wait()
+
+        Dump 'count'
+
+        >>> count = io.gdb.parse_and_eval('$rdx')
+        >>> long = io.gdb.lookup_type('long')
+        >>> int(count.cast(long))
+        4
+
+        Resume the program
+
+        >>> io.gdb.continue_nowait()
+        >>> io.recvline()
+        b'foo\n'
+
+
+    Using SSH:
+
+        You can use :func:`debug` to spawn new processes on remote machines as well,
+        by using the ``ssh=`` keyword to pass in your :class:`.ssh` instance.
+
+        Connect to the SSH server and start a process on the server
+
+        >>> shell = ssh('travis', 'example.pwnme', password='demopass')
+        >>> io = gdb.debug(['bash'],
+        ...                 ssh = shell,
+        ...                 gdbscript = '''
+        ... break main
+        ... continue
+        ... ''')
+
+        Send a command to Bash
+
+        >>> io.sendline("echo hello")
+
+        Interact with the process
+        >>> io.interactive() # doctest: +SKIP
+        >>> io.close()
     """
     if isinstance(args, six.integer_types + (tubes.process.process, tubes.ssh.ssh_channel)):
         log.error("Use gdb.attach() to debug a running process")
@@ -415,6 +521,9 @@ def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, **kw
     which  = _get_which(ssh)
     gdbscript = gdbscript or ''
 
+    if api and runner is not tubes.process.process:
+        raise ValueError('GDB Python API is supported only for local processes')
+
     if context.noptrace:
         log.warn_once("Skipping debugger since context.noptrace==True")
         return runner(args, executable=exe, env=env)
@@ -427,7 +536,10 @@ def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, **kw
         sysroot = sysroot or qemu.ld_prefix(env=env)
         if not qemu_user:
             log.error("Cannot debug %s binaries without appropriate QEMU binaries" % context.arch)
-        qemu_args = [qemu_user, '-g', str(qemu_port)]
+        if context.os == 'baremetal':
+            qemu_args = [qemu_user, '-S', '-gdb', 'tcp::' + str(qemu_port)]
+        else:
+            qemu_args = [qemu_user, '-g', str(qemu_port)]
         if sysroot:
             qemu_args += ['-L', sysroot]
         args = qemu_args + args
@@ -462,7 +574,10 @@ def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, **kw
     if not ssh and context.os == 'android':
         host = context.adb_host
 
-    attach((host, port), exe=exe, gdbscript=gdbscript, ssh=ssh, sysroot=sysroot)
+    tmp = attach((host, port), exe=exe, gdbscript=gdbscript, ssh=ssh, sysroot=sysroot, api=api)
+    if api:
+        _, gdb = tmp
+        gdbserver.gdb = gdb
 
     # gdbserver outputs a message when a client connects
     garbage = gdbserver.recvline(timeout=1)
@@ -509,8 +624,97 @@ def binary():
 
     return gdb
 
+class Breakpoint:
+    """Mirror of ``gdb.Breakpoint`` class.
+
+    See https://sourceware.org/gdb/onlinedocs/gdb/Breakpoints-In-Python.html
+    for more information.
+    """
+
+    def __init__(self, conn, *args, **kwargs):
+        """Do not create instances of this class directly.
+
+        Use ``pwnlib.gdb.Gdb.Breakpoint`` instead.
+        """
+        # Creates a real breakpoint and connects it with this mirror
+        self.conn = conn
+        self.server_breakpoint = conn.root.set_breakpoint(
+            self, hasattr(self, 'stop'), *args, **kwargs)
+
+    def __getattr__(self, item):
+        """Return attributes of the real breakpoint."""
+        if item in (
+                '____id_pack__',
+                '__name__',
+                '____conn__',
+                'stop',
+        ):
+            # Ignore RPyC netref attributes.
+            # Also, if stop() is not defined, hasattr() call in our
+            # __init__() will bring us here. Don't contact the
+            # server in this case either.
+            raise AttributeError()
+        return getattr(self.server_breakpoint, item)
+
+    def exposed_stop(self):
+        # Handle stop() call from the server.
+        return self.stop()
+
+class Gdb:
+    """Mirror of ``gdb`` module.
+
+    See https://sourceware.org/gdb/onlinedocs/gdb/Basic-Python.html for more
+    information.
+    """
+
+    def __init__(self, conn):
+        """Do not create instances of this class directly.
+
+        Use :func:`attach` or :func:`debug` with ``api=True`` instead.
+        """
+        self.conn = conn
+
+        class _Breakpoint(Breakpoint):
+            def __init__(self, *args, **kwargs):
+                super().__init__(conn, *args, **kwargs)
+
+        self.Breakpoint = _Breakpoint
+        self.stopped = Event()
+
+        def stop_handler(event):
+            self.stopped.set()
+
+        self.events.stop.connect(stop_handler)
+
+    def __getattr__(self, item):
+        """Provide access to the attributes of `gdb` module."""
+        return getattr(self.conn.root.gdb, item)
+
+    def wait(self):
+        """Wait until the program stops."""
+        self.stopped.wait()
+        self.stopped.clear()
+
+    def interrupt_and_wait(self):
+        """Interrupt the program and wait until it stops."""
+        self.execute('interrupt')
+        self.wait()
+
+    def continue_nowait(self):
+        """Continue the program. Do not wait until it stops again."""
+        self.execute('continue &')
+
+    def continue_and_wait(self):
+        """Continue the program and wait until it stops again."""
+        self.continue_nowait()
+        self.wait()
+
+    def quit(self):
+        """Terminate GDB."""
+        self.conn.root.quit()
+
 @LocalContext
-def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysroot = None):
+def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysroot = None, api = False):
     r"""
     Start GDB in a new terminal and attach to `target`.
 
@@ -523,9 +727,11 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
         gdb_args(list): List of additional arguments to pass to GDB.
         sysroot(str): Foreign-architecture sysroot, used for QEMU-emulated binaries
             and Android targets.
+        api(bool): Enable access to GDB Python API.
 
     Returns:
         PID of the GDB process (or the window which it is running in).
+        When ``api=True``, a (PID, :class:`Gdb`) tuple.
 
     Notes:
 
@@ -571,6 +777,39 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
         >>> io.recvall()
         b'Hello from bash\n'
 
+        Using GDB Python API:
+
+        .. doctest
+           :skipif: six.PY2
+
+            >>> io = process('bash')
+
+            Attach a debugger
+
+            >>> pid, io_gdb = gdb.attach(io, api=True)
+
+            Force the program to write something it normally wouldn't
+
+            >>> io_gdb.execute('call puts("Hello from process debugger!")')
+
+            Resume the program
+
+            >>> io_gdb.continue_nowait()
+
+            Observe the forced line
+
+            >>> io.recvline()
+            b'Hello from process debugger!\n'
+
+            Interact with the program in a regular way
+
+            >>> io.sendline('echo Hello from bash && exit')
+
+            Observe the results
+
+            >>> io.recvall()
+            b'Hello from bash\n'
+
         Attach to the remote process from a :class:`.remote` or :class:`.listen` tube,
         as long as it is running on the same machine.
 
@@ -590,7 +829,7 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
         b'Hello from bash\n'
 
         Attach to processes running on a remote machine via an SSH :class:`.ssh` process
-        
+
         >>> shell = ssh('travis', 'example.pwnme', password='demopass')
         >>> io = shell.process(['cat'])
         >>> pid = gdb.attach(io, gdbscript='''
@@ -635,7 +874,7 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
         if context.os == 'android':
             pre += 'set gnutarget ' + _bfdname() + '\n'
 
-        if exe:
+        if exe and context.os != 'baremetal':
             pre += 'file %s\n' % exe
 
     # let's see if we can find a pid to attach to
@@ -701,7 +940,7 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
         # before we attach the debugger.
         t = Timeout()
         with t.countdown(2):
-            while exe and os.realpath(proc.exe(pid)) != os.realpath(exe) and t.timeout:
+            while exe and os.path.realpath(proc.exe(pid)) != os.path.realpath(exe) and t.timeout:
                 time.sleep(0.1)
 
     elif isinstance(target, tubes.process.process):
@@ -744,7 +983,8 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
     if not pid and not exe and not ssh:
         log.error('could not find target process')
 
-    cmd = binary()
+    gdb_binary = binary()
+    cmd = gdb_binary
 
     if gdb_args:
         cmd += ' '
@@ -777,6 +1017,17 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
         # when you're trying to debug anything that forks.
         pre += 'set detach-on-fork off\n'
 
+    if api:
+        # create a UNIX socket for talking to GDB
+        socket_dir = tempfile.mkdtemp()
+        socket_path = os.path.join(socket_dir, 'socket')
+        bridge = os.path.join(os.path.dirname(__file__), 'gdb_api_bridge.py')
+
+        # inject the socket path and the GDB Python API bridge
+        pre = 'python socket_path = ' + repr(socket_path) + '\n' + \
+              'source ' + bridge + '\n' + \
+              pre
+
     gdbscript = pre + (gdbscript or '')
 
     if gdbscript:
@@ -791,12 +1042,69 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
 
     log.info('running in new terminal: %s' % cmd)
 
-    gdb_pid = misc.run_in_new_terminal(cmd)
+    if api:
+        # prevent gdb_faketerminal.py from messing up api doctests
+        def preexec_fn():
+            os.environ['GDB_FAKETERMINAL'] = '0'
+    else:
+        preexec_fn = None
+    gdb_pid = misc.run_in_new_terminal(cmd, preexec_fn = preexec_fn)
 
     if pid and context.native:
         proc.wait_for_debugger(pid, gdb_pid)
 
-    return gdb_pid
+    if not api:
+        return gdb_pid
+
+    # connect to the GDB Python API bridge
+    if six.PY2:
+        retriable = socket.error
+    else:
+        retriable = ConnectionRefusedError, FileNotFoundError
+
+    t = Timeout()
+    with t.countdown(10):
+        while t.timeout:
+            try:
+                conn = unix_connect(socket_path)
+                break
+            except retriable:
+                time.sleep(0.1)
+        else:
+            # Check to see if RPyC is installed at all in GDB
+            rpyc_check = [gdb_binary, '--nx', '-batch', '-ex',
+                          'python import rpyc; import sys; sys.exit(123)']
+
+            if 123 != tubes.process.process(rpyc_check).poll(block=True):
+                log.error('Failed to connect to GDB: rpyc is not installed')
+
+            # Check to see if the socket ever got created
+            if not os.path.exists(socket_path):
+                log.error('Failed to connect to GDB: Unix socket %s was never created', socket_path)
+
+            # Check to see if the remote RPyC client is a compatible version
+            version_check = [gdb_binary, '--nx', '-batch', '-ex',
+                            'python import platform; print(platform.python_version())']
+            gdb_python_version = tubes.process.process(version_check).recvall().strip()
+            python_version = str(platform.python_version())
+
+            if gdb_python_version != python_version:
+                log.error('Failed to connect to GDB: Version mismatch (%s vs %s)',
+                           gdb_python_version,
+                           python_version)
+
+            # Don't know what happened
+            log.error('Failed to connect to GDB: Unknown error')
+
+    # now that connection is up, remove the socket from the filesystem
+    os.unlink(socket_path)
+    os.rmdir(socket_dir)
+
+    # create a thread for receiving breakpoint notifications
+    BgServingThread(conn, callback=lambda: None)
+
+    return gdb_pid, Gdb(conn)
+
 
 def ssh_gdb(ssh, argv, gdbscript = None, arch = None, **kwargs):
     if not isinstance(argv, (list, tuple)):
@@ -966,8 +1274,8 @@ def corefile(process):
 
         >>> io = process('bash')
         >>> core = gdb.corefile(io)
-        >>> core.exe.name
-        '/bin/bash'
+        >>> core.exe.name # doctest: +ELLIPSIS
+        '.../bin/bash'
     """
 
     if context.noptrace:
@@ -1022,7 +1330,7 @@ def version(program='gdb'):
 
     Example:
 
-        >>> (7,0) <= gdb.version() <= (10,0)
+        >>> (7,0) <= gdb.version() <= (12,0)
         True
     """
     program = misc.which(program)
