@@ -3,15 +3,17 @@ from __future__ import division
 import base64
 import errno
 import os
-import platform
 import re
+import signal
 import six
 import socket
 import stat
 import string
+import subprocess
+import sys
+import tempfile
 
-import six
-
+from pwnlib import atexit
 from pwnlib.context import context
 from pwnlib.log import getLogger
 from pwnlib.util import fiddling
@@ -131,7 +133,7 @@ def write(path, data = b'', create_dir = False, mode = 'w'):
     with open(path, mode) as f:
         f.write(data)
 
-def which(name, all = False):
+def which(name, all = False, path=None):
     """which(name, flags = os.X_OK, all = False) -> str or str set
 
     Works as the system command ``which``; searches $PATH for ``name`` and
@@ -149,9 +151,10 @@ def which(name, all = False):
       else the first location or :const:`None` if not found.
 
     Example:
-      >>> which('sh')
-      '/bin/sh'
-"""
+
+        >>> which('sh') # doctest: +ELLIPSIS
+        '.../bin/sh'
+    """
     # If name is a path, do not attempt to resolve it.
     if os.path.sep in name:
         return name
@@ -159,7 +162,7 @@ def which(name, all = False):
     isroot = os.getuid() == 0
     out = set()
     try:
-        path = os.environ['PATH']
+        path = path or os.environ['PATH']
     except KeyError:
         log.exception('Environment variable $PATH is not set')
     for p in path.split(os.pathsep):
@@ -181,8 +184,8 @@ def which(name, all = False):
     else:
         return None
 
-def run_in_new_terminal(command, terminal = None, args = None):
-    """run_in_new_terminal(command, terminal = None) -> None
+def run_in_new_terminal(command, terminal=None, args=None, kill_at_exit=True, preexec_fn=None):
+    """run_in_new_terminal(command, terminal=None, args=None, kill_at_exit=True, preexec_fn=None) -> int
 
     Run a command in a new terminal.
 
@@ -190,21 +193,26 @@ def run_in_new_terminal(command, terminal = None, args = None):
         - If ``context.terminal`` is set it will be used.
           If it is an iterable then ``context.terminal[1:]`` are default arguments.
         - If a ``pwntools-terminal`` command exists in ``$PATH``, it is used
-        - If ``$TERM_PROGRAM`` is set, that is used.
-        - If X11 is detected (by the presence of the ``$DISPLAY`` environment
-          variable), ``x-terminal-emulator`` is used.
         - If tmux is detected (by the presence of the ``$TMUX`` environment
           variable), a new pane will be opened.
         - If GNU Screen is detected (by the presence of the ``$STY`` environment
           variable), a new screen will be opened.
+        - If ``$TERM_PROGRAM`` is set, that is used.
+        - If X11 is detected (by the presence of the ``$DISPLAY`` environment
+          variable), ``x-terminal-emulator`` is used.
         - If WSL (Windows Subsystem for Linux) is detected (by the presence of
           a ``wsl.exe`` binary in the ``$PATH`` and ``/proc/sys/kernel/osrelease``
           containing ``Microsoft``), a new ``cmd.exe`` window will be opened.
 
+    If `kill_at_exit` is :const:`True`, try to close the command/terminal when the
+    current process exits. This may not work for all terminal types.
+
     Arguments:
         command (str): The command to run.
         terminal (str): Which terminal to use.
         args (list): Arguments to pass to the terminal
+        kill_at_exit (bool): Whether to close the command/terminal on process exit.
+        preexec_fn (callable): Callable to invoke before exec().
 
     Note:
         The command is opened with ``/dev/null`` for stdin, stdout, stderr.
@@ -248,6 +256,12 @@ def run_in_new_terminal(command, terminal = None, args = None):
     if isinstance(args, tuple):
         args = list(args)
 
+    # When not specifying context.terminal explicitly, we used to set these flags above.
+    # However, if specifying terminal=['tmux', 'splitw', '-h'], we would be lacking these flags.
+    # Instead, set them here and hope for the best.
+    if terminal == 'tmux':
+        args += ['-F' '#{pane_pid}', '-P']
+
     argv = [which(terminal)] + args
 
     if isinstance(command, six.string_types):
@@ -255,23 +269,49 @@ def run_in_new_terminal(command, terminal = None, args = None):
             log.error("Cannot use commands with semicolon.  Create a script and invoke that directly.")
         argv += [command]
     elif isinstance(command, (list, tuple)):
-        if any(';' in c for c in command):
-            log.error("Cannot use commands with semicolon.  Create a script and invoke that directly.")
-        argv += list(command)
+        # Dump the full command line to a temporary file so we can be sure that
+        # it is parsed correctly, and we do not need to account for shell expansion
+        script = '''
+#!{executable!s}
+import os
+os.execve({argv0!r}, {argv!r}, os.environ)
+'''
+        script = script.format(executable=sys.executable,
+                               argv=command,
+                               argv0=which(command[0]))
+        script = script.lstrip()
+
+        log.debug("Created script for new terminal:\n%s" % script)
+
+        with tempfile.NamedTemporaryFile(delete=False, mode='wt+') as tmp:
+          tmp.write(script)
+          tmp.flush()
+          os.chmod(tmp.name, 0o700)
+          argv += [tmp.name]
+
 
     log.debug("Launching a new terminal: %r" % argv)
 
-    pid = os.fork()
+    stdin = stdout = stderr = open(os.devnull, 'r+b')
+    if terminal == 'tmux':
+        stdout = subprocess.PIPE
 
-    if pid == 0:
-        # Closing the file descriptors makes everything fail under tmux on OSX.
-        if platform.system() != 'Darwin':
-            devnull = open(os.devnull, 'r+b')
-            os.dup2(devnull.fileno(), 0)
-            os.dup2(devnull.fileno(), 1)
-            os.dup2(devnull.fileno(), 2)
-        os.execv(argv[0], argv)
-        os._exit(1)
+    p = subprocess.Popen(argv, stdin=stdin, stdout=stdout, stderr=stderr, preexec_fn=preexec_fn)
+
+    if terminal == 'tmux':
+        out, _ = p.communicate()
+        pid = int(out)
+    else:
+        pid = p.pid
+
+    if kill_at_exit:
+        def kill():
+            try:
+                os.kill(pid, signal.SIGTERM)
+            except OSError:
+                pass
+
+        atexit.register(kill)
 
     return pid
 
